
---
# 4G LTE 3 E Click

> 4G LTE 3 Click (for Europe) is a compact add-on board for reliable LTE connectivity, offering medium data speeds and extensive network coverage. This board features the LEXI-R10801D, a single-mode LTE Cat 1bis module from u-blox optimized for professional-grade applications in the EMEA, APAC, and Brazilian regions. The module supports various LTE FDD bands (1, 3, 5, 7, 8, 20, and 28), features download speeds of up to 10Mbps and upload speeds of 5Mbps, and integrates an embedded Wi-Fi scan for indoor positioning with u-blox CellLocate® geolocation capabilities. It includes a UART interface for communication, USB Type C for power and firmware updates, and versatile control options like GPIO pins and visual indicators for network and power status.

<p align="center">
  <img src="https://download.mikroe.com/images/click_for_ide/4glte3_click.png" height=300px>
</p>

[Click Product page](https://www.mikroe.com/4g-lte-3-click-for-europe)

---


#### Click library

- **Author**        : Stefan Filipovic
- **Date**          : Sep 2024.
- **Type**          : UART type


# Software Support

We provide a library for the 4G LTE 3 E Click
as well as a demo application (example), developed using MikroElektronika
[compilers](https://www.mikroe.com/necto-studio).
The demo can run on all the main MikroElektronika [development boards](https://www.mikroe.com/development-boards).

Package can be downloaded/installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

## Library Description

> This library contains API for 4G LTE 3 E Click driver.

#### Standard key functions :

- `c4glte3e_cfg_setup` Config Object Initialization function.
```c
void c4glte3e_cfg_setup ( c4glte3e_cfg_t *cfg );
```

- `c4glte3e_init` Initialization function.
```c
err_t c4glte3e_init ( c4glte3e_t *ctx, c4glte3e_cfg_t *cfg );
```

#### Example key functions :

- `c4glte3e_set_sim_apn` This function sets APN for sim card.
```c
void c4glte3e_set_sim_apn ( c4glte3e_t *ctx, uint8_t *sim_apn );
```

- `c4glte3e_send_sms_text` This function sends text message to a phone number.
```c
void c4glte3e_send_sms_text ( c4glte3e_t *ctx, uint8_t *phone_number, uint8_t *sms_text );
```

- `c4glte3e_cmd_run` This function sends a specified command to the Click module.
```c
void c4glte3e_cmd_run ( c4glte3e_t *ctx, uint8_t *cmd );
```

## Example Description

> Application example shows device capability of connecting to the network and sending SMS or TCP/UDP messages using standard "AT" commands.

**The demo application is composed of two sections :**

### Application Init

> Initializes the driver and logger.

```c

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    c4glte3e_cfg_t c4glte3e_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    c4glte3e_cfg_setup( &c4glte3e_cfg );
    C4GLTE3E_MAP_MIKROBUS( c4glte3e_cfg, MIKROBUS_1 );
    if ( UART_ERROR == c4glte3e_init( &c4glte3e, &c4glte3e_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    log_info( &logger, " Application Task " );

    app_state = C4GLTE3E_POWER_UP;
    log_printf( &logger, ">>> APP STATE - POWER UP <<<\r\n\n" );
}

```

### Application Task

> Application task is split in few stages:
 - C4GLTE3E_POWER_UP:
   > Powers up the device, performs a factory reset and reads system information.
 - C4GLTE3E_CONFIG_CONNECTION:
   > Sets configuration to device to be able to connect to the network.
 - C4GLTE3E_CHECK_CONNECTION:
   > Waits for the network registration indicated via CEREG command and then checks the signal quality report.
 - C4GLTE3E_CONFIG_EXAMPLE:
   > Configures device for the selected example.
 - C4GLTE3E_EXAMPLE:
   > Depending on the selected demo example, it sends an SMS message (in PDU or TXT mode) or TCP/UDP message.
> By default, the TCP/UDP example is selected.

```c
void application_task ( void )
{
    switch ( app_state )
    {
        case C4GLTE3E_POWER_UP:
        {
            if ( C4GLTE3E_OK == c4glte3e_power_up( &c4glte3e ) )
            {
                app_state = C4GLTE3E_CONFIG_CONNECTION;
                log_printf( &logger, ">>> APP STATE - CONFIG CONNECTION <<<\r\n\n" );
            }
            break;
        }
        case C4GLTE3E_CONFIG_CONNECTION:
        {
            if ( C4GLTE3E_OK == c4glte3e_config_connection( &c4glte3e ) )
            {
                app_state = C4GLTE3E_CHECK_CONNECTION;
                log_printf( &logger, ">>> APP STATE - CHECK CONNECTION <<<\r\n\n" );
            }
            break;
        }
        case C4GLTE3E_CHECK_CONNECTION:
        {
            if ( C4GLTE3E_OK == c4glte3e_check_connection( &c4glte3e ) )
            {
                app_state = C4GLTE3E_CONFIG_EXAMPLE;
                log_printf( &logger, ">>> APP STATE - CONFIG EXAMPLE <<<\r\n\n" );
            }
            break;
        }
        case C4GLTE3E_CONFIG_EXAMPLE:
        {
            if ( C4GLTE3E_OK == c4glte3e_config_example( &c4glte3e ) )
            {
                app_state = C4GLTE3E_EXAMPLE;
                log_printf( &logger, ">>> APP STATE - EXAMPLE <<<\r\n\n" );
            }
            break;
        }
        case C4GLTE3E_EXAMPLE:
        {
            c4glte3e_example( &c4glte3e );
            break;
        }
        default:
        {
            log_error( &logger, " APP STATE." );
            break;
        }
    }
}
```

## Note

> In order for the examples to work, user needs to set the APN and SMSC (SMS PDU mode only)
> of entered SIM card as well as the phone number (SMS mode only) to which he wants to send an SMS.
> Enter valid values for the following macros: SIM_APN, SIM_SMSC and PHONE_NUMBER.
> > Example: 
> > - SIM_APN "internet"
> > - SIM_SMSC "+381610401"
> > - PHONE_NUMBER "+381659999999"

The full application code, and ready to use projects can be installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

**Other Mikroe Libraries used in the example:**

- MikroSDK.Board
- MikroSDK.Log
- Click.4GLTE3E

**Additional notes and informations**

Depending on the development board you are using, you may need
[USB UART Click](https://www.mikroe.com/usb-uart-click),
[USB UART 2 Click](https://www.mikroe.com/usb-uart-2-click) or
[RS232 Click](https://www.mikroe.com/rs232-click) to connect to your PC, for
development systems with no UART to USB interface available on the board. UART
terminal is available in all MikroElektronika
[compilers](https://shop.mikroe.com/compilers).

---
